# CMake file
# Copyright (C) ZIH, Technische Universitaet Dresden, Federal Republic of Germany, 2011-2012
# Copyright (C) Lawrence Livermore National Laboratories, United States of America, 2011-2012
#
# See the file LICENSE.txt in the package base directory for details
#
# @file FindGTI.cmake
#       Detects the GTI installation.
#
# @author Tobias Hilbrich
# @date 03.01.2011

SET(GTI_FOUND FALSE)

## TODO:
## In future this might be enhanced by provinding a "gti-config" tool that can be added to the bin path and queried by 
## this CMake file.
SET (GTI_HOME "" CACHE PATH "GTI installation directory.")

IF (EXISTS "${GTI_HOME}/cmakemodules/InstallationConfiguration.cmake")
    #Was found
    SET(GTI_FOUND TRUE)
    
    #Add the helper modules from GTI, and the installation configuration of it
    SET(CMAKE_MODULE_PATH "${GTI_HOME}/cmakemodules;${CMAKE_MODULE_PATH}")
    INCLUDE (HelperMacros)
    INCLUDE (InstallationConfiguration)
    SET(CMAKE_MODULE_PATH "${PnMPI_HOME}/share/cmake/PnMPI;${CMAKE_MODULE_PATH}")
    INCLUDE (PnMPIConfig)
    INCLUDE (InstallationConfiguration)
    
    #Set important filepathes and pathes for GTI components    
    SET (WEAVER_EXE "${GTI_HOME}/bin/weaver")
    SET (WRAPPGEN_EXE "${GTI_HOME}/bin/wrappgen")
    SET (RECVGEN_EXE "${GTI_HOME}/bin/recvgen")
    SET (BUILDGEN_EXE "${GTI_HOME}/bin/buildgen")
    SET (MODCONFGEN_EXE "${GTI_HOME}/bin/modconfgen")
    SET (GTI_SPECIFICATION "${GTI_HOME}/specifications/gti-specification.xml")
    
    SET (GTI_INC_DIR "${GTI_HOME}/include")
    SET (GTI_SRC_DIR "${GTI_HOME}/src")
    SET (GTI_MODULE_DIR "${GTI_HOME}/modules")
    
 ELSE (EXISTS "${GTI_HOME}/cmakemodules/InstallationConfiguration.cmake")
    #Was not found
    IF (GTI_FIND_REQUIRED)
        MESSAGE (FATAL_ERROR "GTI installation was not found, please specify GTI_HOME.")
    ELSE ()
        MESSAGE ("GTI installation was not found, please specify GTI_HOME.")
    ENDIF ()
ENDIF (EXISTS "${GTI_HOME}/cmakemodules/InstallationConfiguration.cmake")
