/*
 * MUST (Marmot Umpire Scalable Tool)
 * Copyright (C) ZIH, Technische Universitaet Dresden, Federal Republic of Germany, 2011-2012
 * Copyright (C) Lawrence Livermore National Laboratories, United States of America, 2011-2012
 *
 * See the file LICENSE.txt in the package base directory for details
 */

/**
 * @file ResourceApi.h
 * 		P call definition for MUST resource track API calls.
 *
 * @author Tobias Hilbrich
 * @date 22.02.2011
 */

#ifndef RESSOURCE_API_H
#define RESSOURCE_API_H

#include "I_DatatypeTrack.h"
#include "I_CommTrack.h"

//==Function used for propagateing predefined datatypes
inline int PpropagatePredefinedDatatypes (
		MustDatatypeType datatypeNull,
		int numTypes,
		int* enumValues,
		MustDatatypeType* handleValues,
		MustAddressType *extents,
		int *alignments) {return 0;}

typedef int (*propagatePredefinedDatatypesP) (
		MustDatatypeType datatypeNull,
		int numTypes,
		int* enumValues,
		MustDatatypeType* handleValues,
		MustAddressType *extents,
		int *alignments);

//==Function used for propagating predefined communicators
inline int PpropagateComms (
		int reachableBegin,
		int reachableEnd,
		int worldSize,
		MustCommType commNull,
		MustCommType commSelf,
		MustCommType commWorld,
		int numWorlds,
		MustCommType* worlds) {return 0;}

typedef int (*propagateCommsP) (
		int reachableBegin,
		int reachableEnd,
		int worldSize,
		MustCommType commNull,
		MustCommType commSelf,
		MustCommType commWorld,
		int numWorlds,
		MustCommType* worlds);

//==Functions to free ressources that where passed across
inline int PpassFreeCommAcross (
        int rank,
        MustRemoteIdType remoteId
        ) {return 0;}

inline int PpassFreeDatatypeAcross (
        int rank,
        MustRemoteIdType remoteId
        ) {return 0;}

inline int PpassFreeErrAcross (
        int rank,
        MustRemoteIdType remoteId
        ) {return 0;}

inline int PpassFreeGroupAcross (
        int rank,
        MustRemoteIdType remoteId
        ) {return 0;}

inline int PpassFreeGroupTableAcross (
        int rank,
        MustRemoteIdType remoteId
        ) {return 0;}

inline int PpassFreeKeyvalAcross (
        int rank,
        MustRemoteIdType remoteId
        ) {return 0;}

inline int PpassFreeOpAcross (
        int rank,
        MustRemoteIdType remoteId
        ) {return 0;}

inline int PpassFreeRequestAcross (
        int rank,
        MustRemoteIdType remoteId
        ) {return 0;}

//Function pointer to any of the above
typedef int (*passFreeAcrossP) (
        int rank,
        MustRemoteIdType remoteId,
        int toPlace
        );

//==Dummy P-Function
inline int PpassCommAcross (
        int rank,
        int hasHandle,
        MustCommType commHandle,
        MustRemoteIdType remoteId,

        int isNull,
        int isPredefined,
        int predefinedEnum, /*MustMpiCommPredefined*/

        //Only for user communicators (isKnown && !isNull && !isPredefined)
        int isCartesian,
        int isGraph,
        int isIntercomm,

        //Only for user and predefined comms (isKnown && !isNull)
        unsigned long long contextId,
        /*unsigned long long myNextContextId;*/ //Not needed, remote side can't derive from this

        MustRemoteIdType groupTableId, //Table information is sent seperately

        //Only for inter-communicators
        MustRemoteIdType groupTableIdRemote, /**< The remote group of the intercommunicator.*/

        //Only for user defined comms
        MustParallelId creationPId,
        MustLocationId creationLId,

        //For cartesian or graph comms
        int reorder,

        //Only for cartesian comms
        int ndims,
        int *dims,
        int *periods,

        //Only for graph comms
        int nnodes,
        int nedges,
        int *indices,
        int *edges) {return 0;}

//==Function pointer used to pass a communicator to a different place on the same level
typedef int (*passCommAcrossP) (
        int rank,
        int hasHandle,
        MustCommType commHandle,
        MustRemoteIdType remoteId,
        int isNull,
        int isPredefined,
        int predefinedEnum,
        int isCartesian,
        int isGraph,
        int isIntercomm,
        unsigned long long contextId,
        MustRemoteIdType groupTableId,
        MustRemoteIdType groupTableIdRemte,
        MustParallelId creationPId,
        MustLocationId creationLId,
        int reorder,
        int ndims,
        int *dims,
        int *periods,
        int nnodes,
        int nedges,
        int *indices,
        int *edges,
        int toPlaceId
        );

//== Dummy P-Functions for passing group tables across
inline int PpassGroupTableAcrossRep2 (
        int rank,
        MustRemoteIdType remoteId,
        int size,
        int* translation
        ) {return 0;}

inline int PpassGroupTableAcrossRep1 (
        int rank,
        MustRemoteIdType remoteId,
        int beginRank,
        int endRank
        ) {return 0;}

//==Function pointers used to pass a group table to a different place on the same level
typedef int (*passGroupTableAcrossRep2P) (
        int rank,
        MustRemoteIdType remoteId,
        int size,
        int* translation,
        int toPlaceId
        );

typedef int (*passGroupTableAcrossRep1P) (
        int rank,
        MustRemoteIdType remoteId,
        int beginRank,
        int endRank,
        int toPlaceId
        );

//== Dummy P-Functions for passing a request across
inline int PpassRequestAcross (
        int rank,
        int hasHandle,
        MustRequestType requestHandle,
        MustRemoteIdType remoteId,
        int isActive,
        int isPersistent,
        int isSend,
        int isNull,
        int isCanceled,
        int isProcNull,
        int count,
        MustRemoteIdType datatype,
        int tag,
        MustRemoteIdType comm,
        int destSource,
        int sendMode,
        MustParallelId creationPId,
        MustParallelId activationPId,
        MustParallelId cancelPId,
        MustLocationId creationLId,
        MustLocationId activationLId,
        MustLocationId cancelLId
        ) {return 0;}

//==Function pointer used to pass a request to a different place on the same level
typedef int (*passRequestAcrossP) (
        int rank,
        int hasHandle,
        MustRequestType requestHandle,
        MustRemoteIdType remoteId,
        int isActive,
        int isPersistent,
        int isSend,
        int isNull,
        int isCanceled,
        int isProcNull,
        int count,
        MustRemoteIdType datatype,
        int tag,
        MustRemoteIdType comm,
        int destSource,
        int sendMode,
        MustParallelId creationPId,
        MustParallelId activationPId,
        MustParallelId cancelPId,
        MustLocationId creationLId,
        MustLocationId activationLId,
        MustLocationId cancelLId,
        int toPlaceId
        );

#endif /*RESSOURCE_API_H*/

